from __future__ import absolute_import, division, print_function

import json

from jinja2 import Environment, Undefined
from markupsafe import Markup

GRAPHIQL_VERSION = '0.10.2'

TEMPLATE = '''<!--
The request to this GraphQL server provided the header "Accept: text/html"
and as a result has been presented GraphiQL - an in-browser IDE for
exploring GraphQL.
If you wish to receive JSON, provide the header "Accept: application/json" or
add "&raw" to the end of the URL within a browser.
-->
<!DOCTYPE html>
<html>
<head>
  <title>{{graphiql_html_title|default("GraphiQL", true)}}</title>
  <style>
    html, body {
      height: 100%;
      margin: 0;
      overflow: hidden;
      width: 100%;
    }
  </style>
  <meta name="referrer" content="no-referrer">
  <link href="//cdn.jsdelivr.net/graphiql/{{graphiql_version}}/graphiql.css" rel="stylesheet" />
  <script src="//cdn.jsdelivr.net/fetch/0.9.0/fetch.min.js"></script>
  <script src="//cdn.jsdelivr.net/react/15.0.0/react.min.js"></script>
  <script src="//cdn.jsdelivr.net/react/15.0.0/react-dom.min.js"></script>
  <script src="//cdn.jsdelivr.net/graphiql/{{graphiql_version}}/graphiql.min.js"></script>
</head>
<body>
  <script>
    // Collect the URL parameters
    var parameters = {};
    window.location.search.substr(1).split('&').forEach(function (entry) {
      var eq = entry.indexOf('=');
      if (eq >= 0) {
        parameters[decodeURIComponent(entry.slice(0, eq))] =
          decodeURIComponent(entry.slice(eq + 1));
      }
    });
    // Produce a Location query string from a parameter object.
    function locationQuery(params) {
      return '?' + Object.keys(params).map(function (key) {
        return encodeURIComponent(key) + '=' +
          encodeURIComponent(params[key]);
      }).join('&');
    }
    // Derive a fetch URL from the current URL, sans the GraphQL parameters.
    var graphqlParamNames = {
      query: true,
      variables: true,
      operationName: true
    };
    var otherParams = {};
    for (var k in parameters) {
      if (parameters.hasOwnProperty(k) && graphqlParamNames[k] !== true) {
        otherParams[k] = parameters[k];
      }
    }
    var fetchURL = locationQuery(otherParams);
    // Defines a GraphQL fetcher using the fetch API.
    function graphQLFetcher(graphQLParams) {
      return fetch(fetchURL, {
        method: 'post',
        headers: {
          'Accept': 'application/json',
          'Content-Type': 'application/json'
        },
        body: JSON.stringify(graphQLParams),
        credentials: 'include',
      }).then(function (response) {
        return response.text();
      }).then(function (responseBody) {
        try {
          return JSON.parse(responseBody);
        } catch (error) {
          return responseBody;
        }
      });
    }
    // When the query and variables string is edited, update the URL bar so
    // that it can be easily shared.
    function onEditQuery(newQuery) {
      parameters.query = newQuery;
      updateURL();
    }
    function onEditVariables(newVariables) {
      parameters.variables = newVariables;
      updateURL();
    }
    function onEditOperationName(newOperationName) {
      parameters.operationName = newOperationName;
      updateURL();
    }
    function updateURL() {
      history.replaceState(null, null, locationQuery(parameters));
    }
    // Render <GraphiQL /> into the body.
    ReactDOM.render(
      React.createElement(GraphiQL, {
        fetcher: graphQLFetcher,
        onEditQuery: onEditQuery,
        onEditVariables: onEditVariables,
        onEditOperationName: onEditOperationName,
        query: {{ query|tojson }},
        response: {{ result|tojson }},
        variables: {{ variables|tojson }},
        operationName: {{ operation_name|tojson }},
      }),
      document.body
    );
  </script>
</body>
</html>'''

_slash_escape = '\\/' not in json.dumps('/')
jinja_options = {
    'extensions': ['jinja2.ext.autoescape', 'jinja2.ext.with_']
}


# TODO Memoize
def create_jinja_environment():
    options = dict(jinja_options)
    options['autoescape'] = True
    rv = Environment()
    rv.filters['tojson'] = tojson_filter
    return rv


def tojson_filter(obj, **kwargs):
    if isinstance(obj, Undefined):
        return str(obj)
    return Markup(htmlsafe_dumps(obj, **kwargs))


def htmlsafe_dumps(obj, **kwargs):
    """Works exactly like :func:`dumps` but is safe for use in ``<script>``
    tags.  It accepts the same arguments and returns a JSON string.  Note that
    this is available in templates through the ``|tojson`` filter which will
    also mark the result as safe.  Due to how this function escapes certain
    characters this is safe even if used outside of ``<script>`` tags.

    The following characters are escaped in strings:

    -   ``<``
    -   ``>``
    -   ``&``
    -   ``'``

    This makes it safe to embed such strings in any place in HTML with the
    notable exception of double quoted attributes.  In that case single
    quote your attributes or HTML escape it in addition.

    .. versionchanged:: 0.10
       This function's return value is now always safe for HTML usage, even
       if outside of script tags or if used in XHTML.  This rule does not
       hold true when using this function in HTML attributes that are double
       quoted.  Always single quote attributes if you use the ``|tojson``
       filter.  Alternatively use ``|tojson|forceescape``.
    """
    rv = json.dumps(obj, **kwargs) \
        .replace(u'<', u'\\u003c') \
        .replace(u'>', u'\\u003e') \
        .replace(u'&', u'\\u0026') \
        .replace(u"'", u'\\u0027')
    if not _slash_escape:
        rv = rv.replace('\\/', '/')
    return rv


def render_graphiql(query, variables, operation_name, result, graphiql_version=None,
                    graphiql_template=None, graphiql_html_title=None):
    graphiql_version = graphiql_version or GRAPHIQL_VERSION
    template = graphiql_template or TEMPLATE

    jinja = create_jinja_environment()
    context = dict(
        graphiql_version=graphiql_version,
        graphiql_html_title=graphiql_html_title,
        result=result,
        query=query,
        variables=variables,
        operation_name=operation_name
    )

    tmpl = jinja.from_string(template)
    return tmpl.render(context)



from __future__ import absolute_import, division, print_function

from graphql.type.definition import GraphQLArgument, GraphQLField, GraphQLNonNull, GraphQLObjectType
from graphql.type.scalars import GraphQLString
from graphql.type.schema import GraphQLSchema
from tornado.gen import coroutine, Return


@coroutine
def resolve_raises(*_):
    raise Exception("Throws!")


@coroutine
def resolve1(obj, args, context, info):
    raise Return(context.args.get('q'))


@coroutine
def resolve2(obj, args, context, info):
    raise Return(context)


@coroutine
def resolve3(obj, args, context, info):
    raise Return('Hello %s' % (args.get('who') or 'World'))


QueryRootType = GraphQLObjectType(
    name='QueryRoot',
    fields={
        'thrower': GraphQLField(GraphQLNonNull(GraphQLString), resolver=resolve_raises),
        'request': GraphQLField(GraphQLNonNull(GraphQLString),
                                resolver=resolve1),
        'context': GraphQLField(GraphQLNonNull(GraphQLString),
                                resolver=resolve2),
        'test': GraphQLField(
            type=GraphQLString,
            args={
                'who': GraphQLArgument(GraphQLString)
            },
            resolver=resolve3
        )
    }
)

MutationRootType = GraphQLObjectType(
    name='MutationRoot',
    fields={
        'writeTest': GraphQLField(
            type=QueryRootType,
            resolver=lambda *_: QueryRootType
        )
    }
)

Schema = GraphQLSchema(QueryRootType, MutationRootType)



import tornado.web
from tornado.ioloop import IOLoop

from graphene_tornado.schema import schema
from graphene_tornado.tornado_graphql_handler import TornadoGraphQLHandler


class ExampleApplication(tornado.web.Application):

    def __init__(self):
        handlers = [
            (r'/graphql', TornadoGraphQLHandler, dict(graphiql=True, schema=schema)),
            (r'/graphql/batch', TornadoGraphQLHandler, dict(graphiql=True, schema=schema, batch=True)),
            (r'/graphql/graphiql', TornadoGraphQLHandler, dict(graphiql=True, schema=schema))
        ]
        tornado.web.Application.__init__(self, handlers)

if __name__ == '__main__':
    app = ExampleApplication()
    app.listen(5000)
    IOLoop.instance().start()



from __future__ import absolute_import, division, print_function

import json

from six.moves.urllib.parse import urlencode

import pytest
from tornado.escape import to_unicode
from tornado.httpclient import HTTPError

from examples.example import ExampleApplication
from graphene_tornado.tests.http_helper import HttpHelper

GRAPHQL_HEADER = {'Content-Type': 'application/graphql'}
FORM_HEADER = {'Content-Type': 'application/x-www-form-urlencoded'}


@pytest.fixture
def app():
    return ExampleApplication()


@pytest.fixture
def http_helper(http_client, base_url):
    return HttpHelper(http_client, base_url)


@pytest.mark.gen_test
def test_allows_get_with_query_param(http_helper):
    response = yield http_helper.get(url_string(query='{test}'), headers=GRAPHQL_HEADER)

    assert response.code == 200
    assert response_json(response) == {
        'data': {'test': "Hello World"}
    }


@pytest.mark.gen_test
def test_allows_get_with_variable_values(http_helper):
    response = yield http_helper.get(url_string(
        query='query helloWho($who: String){ test(who: $who) }',
        variables=json.dumps({'who': "Dolly"})
    ), headers=GRAPHQL_HEADER)

    assert response.code == 200
    assert response_json(response) == {
        'data': {'test': "Hello Dolly"}
    }


@pytest.mark.gen_test
def test_allows_get_with_operation_name(http_helper):
    response = yield http_helper.get(url_string(
        query='''
            query helloYou { test(who: "You"), ...shared }
            query helloWorld { test(who: "World"), ...shared }
            query helloDolly { test(who: "Dolly"), ...shared }
            fragment shared on QueryRoot {
              shared: test(who: "Everyone")
            }
            ''',
        operationName='helloWorld'
    ), headers=GRAPHQL_HEADER)

    assert response.code == 200
    assert response_json(response) == {
        'data': {
            'test': 'Hello World',
            'shared': 'Hello Everyone'
        }
    }


@pytest.mark.gen_test
def test_reports_validation_errors(http_helper):
    with pytest.raises(HTTPError) as context:
        yield http_helper.get(url_string(
            query='{ test, unknownOne, unknownTwo }'
        ), headers=GRAPHQL_HEADER)

    assert context.value.code == 400
    assert response_json(context.value.response) == {
        'errors': [
            {
                'message': 'Cannot query field "unknownOne" on type "QueryRoot".',
                'locations': [{'line': 1, 'column': 9}]
            },
            {
                'message': 'Cannot query field "unknownTwo" on type "QueryRoot".',
                'locations': [{'line': 1, 'column': 21}]
            }
        ]
    }


@pytest.mark.gen_test
def test_errors_when_missing_operation_name(http_helper):
    with pytest.raises(HTTPError) as context:
        yield http_helper.get(url_string(
            query='''
                query TestQuery { test }
                mutation TestMutation { writeTest { test } }
                '''
        ))

    assert context.value.code == 400
    assert response_json(context.value.response) == {
        'errors': [
            {
                'message': 'Must provide operation name if query contains multiple operations.'
            }
        ]
    }


@pytest.mark.gen_test
def test_errors_when_sending_a_mutation_via_get(http_helper):
    with pytest.raises(HTTPError) as context:
        yield http_helper.get(url_string(
            query='''
                mutation TestMutation { writeTest { test } }
                '''
        ), headers=GRAPHQL_HEADER)

    assert context.value.code == 405
    assert response_json(context.value.response) == {
        'errors': [
            {
                'message': 'Can only perform a mutation operation from a POST request.'
            }
        ]
    }


@pytest.mark.gen_test
def test_errors_when_selecting_a_mutation_within_a_get(http_helper):
    with pytest.raises(HTTPError) as context:
        yield http_helper.get(url_string(
            query='''
                query TestQuery { test }
                mutation TestMutation { writeTest { test } }
                ''',
            operationName='TestMutation'
        ), headers=GRAPHQL_HEADER)

    assert context.value.code == 405
    assert response_json(context.value.response) == {
        'errors': [
            {
                'message': 'Can only perform a mutation operation from a POST request.'
            }
        ]
    }


@pytest.mark.gen_test
def test_allows_mutation_to_exist_within_a_get(http_helper):
    response = yield http_helper.get(url_string(
        query='''
            query TestQuery { test }
            mutation TestMutation { writeTest { test } }
            ''',
        operationName='TestQuery'
    ), headers=GRAPHQL_HEADER)

    assert response.code == 200
    assert response_json(response) == {
        'data': {'test': "Hello World"}
    }


@pytest.mark.gen_test
def test_allows_post_with_json_encoding(http_helper):
    response = yield http_helper.post_json(url_string(), dict(query='{test}'))

    assert response.code == 200
    assert response_json(response) == {
        'data': {'test': "Hello World"}
    }


@pytest.mark.gen_test
def test_batch_allows_post_with_json_encoding(http_helper):
    response = yield http_helper.post_json(batch_url_string(), [dict(id=1, query='{test}')])

    assert response.code == 200
    assert response_json(response) == [{
        'id': 1,
        'data': {'test': "Hello World"},
        'status': 200,
    }]


@pytest.mark.gen_test
def test_batch_fails_if_is_empty(http_helper):
    with pytest.raises(HTTPError) as context:
        yield http_helper.post_body(batch_url_string(), body='[]', headers={'Content-Type': 'application/json'})

    assert context.value.code == 400
    assert response_json(context.value.response) == {
        'errors': [{'message': 'Received an empty list in the batch request.'}]
    }


@pytest.mark.gen_test
def test_allows_sending_a_mutation_via_post(http_helper):
    response = yield http_helper.post_json(url_string(), dict(query='mutation TestMutation { writeTest { test } }'))

    assert response.code == 200
    assert response_json(response) == {
        'data': {'writeTest': {'test': 'Hello World'}}
    }


@pytest.mark.gen_test
def test_allows_post_with_url_encoding(http_helper):
    response = yield http_helper.post_body(url_string(), body=urlencode(dict(query='{test}')), headers=FORM_HEADER)

    assert response.code == 200
    assert response_json(response) == {
        'data': {'test': "Hello World"}
    }


@pytest.mark.gen_test
def test_supports_post_json_query_with_string_variables(http_helper):
    response = yield http_helper.post_json(url_string(), dict(
        query='query helloWho($who: String){ test(who: $who) }',
        variables=json.dumps({'who': "Dolly"})
    ))

    assert response.code == 200
    assert response_json(response) == {
        'data': {'test': "Hello Dolly"}
    }


@pytest.mark.gen_test
def test_batch_supports_post_json_query_with_string_variables(http_helper):
    response = yield http_helper.post_json(batch_url_string(), [dict(
        id=1,
        query='query helloWho($who: String){ test(who: $who) }',
        variables={'who': "Dolly"}
    )])

    assert response.code == 200
    assert response_json(response) == [{
        'id': 1,
        'data': {'test': "Hello Dolly"},
        'status': 200,
    }]


@pytest.mark.gen_test
def test_supports_post_json_query_with_json_variables(http_helper):
    response = yield http_helper.post_json(url_string(), dict(
        query='query helloWho($who: String){ test(who: $who) }',
        variables={'who': "Dolly"}
    ))

    assert response.code == 200
    assert response_json(response) == {
        'data': {'test': "Hello Dolly"}
    }


@pytest.mark.gen_test
def test_batch_supports_post_json_query_with_json_variables(http_helper):
    response = yield http_helper.post_json(batch_url_string(), [dict(
        id=1,
        query='query helloWho($who: String){ test(who: $who) }',
        variables={'who': "Dolly"}
    )])

    assert response.code == 200
    assert response_json(response) == [{
        'id': 1,
        'data': {'test': "Hello Dolly"},
        'status': 200,
    }]


@pytest.mark.gen_test
def test_supports_post_url_encoded_query_with_string_variables(http_helper):
    response = yield http_helper.post_body(url_string(), body=urlencode(dict(
        query='query helloWho($who: String){ test(who: $who) }',
        variables=json.dumps({'who': "Dolly"})
    )), headers=FORM_HEADER)

    assert response.code == 200
    assert response_json(response) == {
        'data': {'test': "Hello Dolly"}
    }


@pytest.mark.gen_test
def test_supports_post_json_quey_with_get_variable_values(http_helper):
    response = yield http_helper.post_json(url_string(
        variables=json.dumps({'who': "Dolly"})
    ), dict(
        query='query helloWho($who: String){ test(who: $who) }',
    ))

    assert response.code == 200
    assert response_json(response) == {
        'data': {'test': "Hello Dolly"}
    }


@pytest.mark.gen_test
def test_post_url_encoded_query_with_get_variable_values(http_helper):
    response = yield http_helper.post_body(url_string(
        variables=json.dumps({'who': "Dolly"})
    ), body=urlencode(dict(
        query='query helloWho($who: String){ test(who: $who) }',
    )), headers=FORM_HEADER)

    assert response.code == 200
    assert response_json(response) == {
        'data': {'test': "Hello Dolly"}
    }


@pytest.mark.gen_test
def test_supports_post_raw_text_query_with_get_variable_values(http_helper):
    response = yield http_helper.post_body(url_string(
        variables=json.dumps({'who': "Dolly"})
    ),
        body='query helloWho($who: String){ test(who: $who) }',
        headers=GRAPHQL_HEADER
    )

    assert response.code == 200
    assert response_json(response) == {
        'data': {'test': "Hello Dolly"}
    }


@pytest.mark.gen_test
def test_allows_post_with_operation_name(http_helper):
    response = yield http_helper.post_json(url_string(), dict(
        query='''
            query helloYou { test(who: "You"), ...shared }
            query helloWorld { test(who: "World"), ...shared }
            query helloDolly { test(who: "Dolly"), ...shared }
            fragment shared on QueryRoot {
              shared: test(who: "Everyone")
            }
            ''',
        operationName='helloWorld'
    ))

    assert response.code == 200
    assert response_json(response) == {
        'data': {
            'test': 'Hello World',
            'shared': 'Hello Everyone'
        }
    }


@pytest.mark.gen_test
def test_batch_allows_post_with_operation_name(http_helper):
    response = yield http_helper.post_json(batch_url_string(), [dict(
        id=1,
        query='''
            query helloYou { test(who: "You"), ...shared }
            query helloWorld { test(who: "World"), ...shared }
            query helloDolly { test(who: "Dolly"), ...shared }
            fragment shared on QueryRoot {
              shared: test(who: "Everyone")
            }
            ''',
        operationName='helloWorld'
    )])

    assert response.code == 200
    assert response_json(response) == [{
        'id': 1,
        'data': {
            'test': 'Hello World',
            'shared': 'Hello Everyone'
        },
        'status': 200,
    }]


@pytest.mark.gen_test
def test_allows_post_with_get_operation_name(http_helper):
    response = yield http_helper.post_body(url_string(
        operationName='helloWorld'
    ), body='''
        query helloYou { test(who: "You"), ...shared }
        query helloWorld { test(who: "World"), ...shared }
        query helloDolly { test(who: "Dolly"), ...shared }
        fragment shared on QueryRoot {
          shared: test(who: "Everyone")
        }
        ''', headers=GRAPHQL_HEADER)

    assert response.code == 200
    assert response_json(response) == {
        'data': {
            'test': 'Hello World',
            'shared': 'Hello Everyone'
        }
    }


@pytest.mark.gen_test
def test_supports_pretty_printing(http_helper):
    response = yield http_helper.get(url_string(query='{test}', pretty=True), headers=GRAPHQL_HEADER)
    assert to_unicode(response.body) == """{
  "data": {
    "test": "Hello World"
  }
}"""


@pytest.mark.gen_test
def test_supports_pretty_printing_by_request(http_helper):
    response = yield http_helper.get(url_string(query='{test}', pretty='1'), headers=GRAPHQL_HEADER)
    assert to_unicode(response.body) == """{
  "data": {
    "test": "Hello World"
  }
}"""


@pytest.mark.gen_test
def test_handles_field_errors_caught_by_graphql(http_helper):
    response = yield http_helper.get(url_string(query='{thrower}'), headers=GRAPHQL_HEADER)
    assert response.code == 200
    assert response_json(response) == {
        'data': None,
        'errors': [{u'path': [u'thrower'], u'message': u'Throws!', u'locations': [{u'column': 2, u'line': 1}]}]
    }


@pytest.mark.gen_test
def test_handles_syntax_errors_caught_by_graphql(http_helper):
    with pytest.raises(HTTPError) as context:
        yield http_helper.get(url_string(query='syntaxerror'), headers=GRAPHQL_HEADER)
    assert context.value.code == 400
    assert response_json(context.value.response) == {
        'errors': [{'locations': [{'column': 1, 'line': 1}],
                    'message': 'Syntax Error GraphQL request (1:1) '
                               'Unexpected Name "syntaxerror"\n\n1: syntaxerror\n   ^\n'}]
    }


@pytest.mark.gen_test
def test_handles_errors_caused_by_a_lack_of_query(http_helper):
    with pytest.raises(HTTPError) as context:
        yield http_helper.get(url_string(), headers=GRAPHQL_HEADER)

    assert context.value.code == 400
    assert response_json(context.value.response) == {
        'errors': [{'message': 'Must provide query string.'}]
    }


@pytest.mark.gen_test
def test_handles_not_expected_json_bodies(http_helper):
    with pytest.raises(HTTPError) as context:
        yield http_helper.post_body(url_string(), body='[]', headers={'Content-Type': 'application/json'})

    assert context.value.code == 400
    assert response_json(context.value.response) == {
        'errors': [{'message': 'The received data is not a valid JSON query.'}]
    }


@pytest.mark.gen_test
def test_handles_invalid_json_bodies(http_helper):
    with pytest.raises(HTTPError) as context:
        yield http_helper.post_body(url_string(), body='[oh}', headers={'Content-Type': 'application/json'})

    assert context.value.code == 400
    assert response_json(context.value.response) == {
        'errors': [{'message': 'POST body sent invalid JSON.'}]
    }


@pytest.mark.gen_test
def test_handles_incomplete_json_bodies(http_helper):
    with pytest.raises(HTTPError) as context:
        yield http_helper.post_body(url_string(), body='{"query":', headers={'Content-Type': 'application/json'})

    assert context.value.code == 400
    assert response_json(context.value.response) == {
        'errors': [{'message': 'POST body sent invalid JSON.'}]
    }


@pytest.mark.gen_test
def test_handles_plain_post_text(http_helper):
    with pytest.raises(HTTPError) as context:
        yield http_helper.post_body(url_string(
            variables=json.dumps({'who': "Dolly"})
        ),
            body='query helloWho($who: String){ test(who: $who) }',
            headers={'Content-Type': 'text/plain'}
        )

    assert context.value.code == 400
    assert response_json(context.value.response) == {
        'errors': [{'message': 'Must provide query string.'}]
    }


@pytest.mark.gen_test
def test_handles_poorly_formed_variables(http_helper):
    with pytest.raises(HTTPError) as context:
        yield http_helper.get(url_string(
            query='query helloWho($who: String){ test(who: $who) }',
            variables='who:You'
        ), headers=GRAPHQL_HEADER)

    assert context.value.code == 400
    assert response_json(context.value.response) == {
        'errors': [{'message': 'Variables are invalid JSON.'}]
    }


@pytest.mark.gen_test
def test_handles_unsupported_http_methods(http_helper):
    with pytest.raises(HTTPError) as context:
        yield http_helper.put(url_string(query='{test}'), '', headers=GRAPHQL_HEADER)
    assert context.value.code == 405


@pytest.mark.gen_test
def test_passes_request_into_context_request(http_helper):
    response = yield http_helper.get(url_string(query='{request}', q='testing'), headers=GRAPHQL_HEADER)

    assert response.code == 200
    assert response_json(response) == {
        'data': {
            'request': 'testing'
        }
    }


def url_string(string='/graphql', **url_params):
    if url_params:
        string += '?' + urlencode(url_params)
    return string


def batch_url_string(**url_params):
    return url_string('/graphql/batch', **url_params)


def response_json(response):
    return json.loads(to_unicode(response.body))



from __future__ import absolute_import, division, print_function

import json

from six.moves.urllib.parse import urlencode


class HttpHelper:

    def __init__(self, http_client, base_url):
        self.http_client = http_client
        self.base_url = base_url

    def get(self, url, **kwargs):
        return self.http_client.fetch(self.get_url(url), **kwargs)

    def delete(self, url, **kwargs):
        kwargs['method'] = kwargs.get('method', 'DELETE')
        return self.get(url, **kwargs)

    def post(self, url, post_data, **kwargs):
        kwargs['method'] = kwargs.get('method', 'POST')
        kwargs['body'] = urlencode(post_data)
        return self.get(url, **kwargs)

    def post_body(self, url, **kwargs):
        kwargs['method'] = kwargs.get('method', 'POST')
        return self.http_client.fetch(self.get_url(url), **kwargs)

    def post_json(self, url, post_data, **kwargs):
        kwargs['method'] = kwargs.get('method', 'POST')
        kwargs['body'] = json.dumps(post_data)
        kwargs['headers'] = kwargs.get('headers', {})
        kwargs['headers']['Content-Type'] = "application/json"
        return self.get(url, **kwargs)

    def put(self, url, put_data, **kwargs):
        kwargs['method'] = kwargs.get('method', 'PUT')
        return self.post(url, put_data, **kwargs)

    def get_url(self, path):
        return self.base_url + path



from setuptools import find_packages, setup
import ast
import re

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('graphene_tornado/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))

tests_require = [
    'coveralls',
    'mock',
    'pytest==3.10.0',
    'pytest-cov',
    'pytest-tornado'
]

setup(
    name='graphene-tornado',
    version=version,

    description='Graphene Tornado integration',
    long_description=open('README.rst').read(),

    url='https://github.com/graphql-python/graphene-tornado',

    author='Eric Hauser',
    author_email='ewhauser@gmail.com',

    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],

    keywords='api graphql protocol rest relay graphene',

    packages=find_packages(exclude=['tests']),

    install_requires=[
        'six>=1.10.0',
        'graphene>=2.0.1',
        'Jinja2==2.9.6',
        'tornado>=5.1.0',
        'werkzeug==0.12.2'
    ],
    setup_requires=[
        'pytest',
    ],
    tests_require=tests_require,
    extras_require={
        'test': tests_require
    },
    include_package_data=True,
    zip_safe=False,
    platforms='any',
)



from __future__ import absolute_import, division, print_function

import pytest
from tornado.escape import to_unicode

from examples.example import ExampleApplication
from graphene_tornado.tests.http_helper import HttpHelper


class Unset(object):
    pass


@pytest.fixture
def app():
    return ExampleApplication()


@pytest.fixture
def http_helper(http_client, base_url):
    return HttpHelper(http_client, base_url)


@pytest.mark.gen_test
def test_displays_graphiql_with_direct_route(http_helper):
    res = yield http_helper.get('/graphql/graphiql', headers={'Accept': 'text/html'})
    has_graphiql(res)


@pytest.mark.gen_test
def test_displays_graphiql_with_accept_html(http_helper):
    res = yield http_helper.get('/graphql/graphiql', headers={'Accept': 'text/html'})
    has_graphiql(res)


@pytest.mark.gen_test
def test_graphiql_is_enabled(http_helper):
    res = yield http_helper.get('/graphql', headers={'Accept': 'text/html'})
    has_graphiql(res)


@pytest.mark.gen_test
def test_graphiql_default_title(http_helper):
    res = yield http_helper.get('/graphql', headers={'Accept': 'text/html'})
    assert '<title>GraphiQL</title>' in to_unicode(res.body)


@pytest.mark.gen_test
def test_graphiql_renders_pretty(http_helper):
    response = yield http_helper.get('/graphql?query={test}', headers={'Accept': 'text/html'})
    pretty_response = (
        '{\n'
        '  "data": {\n'
        '    "test": "Hello World"\n'
        '  }\n'
        '}'
    ).replace("\"", "\\\"").replace("\n", "\\n")
    assert pretty_response in to_unicode(response.body)


@pytest.mark.gen_test
def test_graphiql_renders_pretty(http_helper):
    response = yield http_helper.get('/graphql?query={test}', headers={'Accept': 'text/html'})
    pretty_response = (
        '{\n'
        '  "data": {\n'
        '    "test": "Hello World"\n'
        '  }\n'
        '}'
    ).replace("\"", "\\\"").replace("\n", "\\n")
    assert pretty_response in to_unicode(response.body)


@pytest.mark.gen_test
def test_handles_empty_vars(http_helper):
    response = yield http_helper.post_json('/graphql', headers={'Accept': 'text/html'}, post_data=dict(
        query="",
        variables=None,
        operationName=""
    ))
    has_graphiql(response)


def has_graphiql(response):
    assert "ReactDOM" in to_unicode(response.body)
    assert response.code == 200



from __future__ import absolute_import, division, print_function

from promise import Promise
from tornado.gen import convert_yielded, multi_future
from tornado.ioloop import IOLoop
from tornado.concurrent import is_future


# https://gist.github.com/isi-gach/daef0b34ec5af6f026af52d593131c64
class TornadoExecutor(object):
    def __init__(self, io_loop=None):
        if io_loop is None:
            io_loop = IOLoop.current()
        self.loop = io_loop
        self.futures = []

    def wait_until_finished(self):
        # if there are futures to wait for
        while self.futures:
            # wait for the futures to finish
            futures = self.futures
            self.futures = []
            self.loop.run_sync(lambda: multi_future(futures))

    def execute(self, fn, *args, **kwargs):
        result = fn(*args, **kwargs)
        if is_future(result):
            future = convert_yielded(result)
            self.futures.append(future)
            return Promise.resolve(future)
        return result



from __future__ import absolute_import, division, print_function

import inspect
import json
import sys
import traceback

import six
from graphql import parse, validate, Source, get_operation_ast, execute
from graphql.error import GraphQLError
from graphql.error import format_error as format_graphql_error
from graphql.execution import ExecutionResult
from tornado import web
from tornado.escape import json_encode, to_unicode
from tornado.gen import coroutine, Return
from tornado.locks import Event
from tornado.log import app_log
from tornado.web import HTTPError
from werkzeug.datastructures import MIMEAccept
from werkzeug.http import parse_accept_header

from graphene_tornado.render_graphiql import render_graphiql
from graphene_tornado.tornado_executor import TornadoExecutor


class ExecutionError(Exception):
    def __init__(self, status_code=400, errors=None):
        self.status_code = status_code
        if errors is None:
            self.errors = []
        else:
            self.errors = [str(e) for e in errors]
        self.message = '\n'.join(self.errors)


class TornadoGraphQLHandler(web.RequestHandler):

    executor = None
    schema = None
    batch = False
    middleware = []
    pretty = False
    root_value = None
    graphiql = False
    graphiql_version = None
    graphiql_template = None
    graphiql_html_title = None

    def initialize(self, schema=None, executor=None, middleware=None, root_value=None, graphiql=False, pretty=False,
                   batch=False):
        super(TornadoGraphQLHandler, self).initialize()
        self.schema = schema
        if middleware is not None:
            self.middleware = list(self.instantiate_middleware(middleware))
        self.executor = executor
        self.root_value = root_value
        self.pretty = pretty
        self.graphiql = graphiql
        self.batch = batch

    @coroutine
    def get(self):
        try:
            yield self.run('get')
        except Exception as ex:
            self.handle_error(ex)

    @coroutine
    def post(self):
        try:
            yield self.run('post')
        except Exception as ex:
            self.handle_error(ex)

    @coroutine
    def run(self, method):
        show_graphiql = self.graphiql and self.should_display_graphiql()
        data = self.parse_body()

        if self.batch:
            responses = []
            for entry in data:
                r = yield self.get_response(entry, method, entry)
                responses.append(r)
            result = '[{}]'.format(','.join([response[0] for response in responses]))
            status_code = max(responses, key=lambda response: response[1])[1]
        else:
            result, status_code = yield self.get_response(data, method, show_graphiql)

        if show_graphiql:
            query, variables, operation_name, id = self.get_graphql_params(self.request, data)
            graphiql = self.render_graphiql(
                query=query or '',
                variables='' if variables is None else json.dumps(variables),
                operation_name=operation_name or '',
                result=result or ''
            )
            self.write(graphiql)
            self.finish()
            return

        self.set_status(status_code)
        self.set_header('Content-Type', 'application/json')
        self.write(result)
        self.finish()

    def parse_body(self):
        content_type = self.content_type

        if content_type == 'application/graphql':
            return {'query': to_unicode(self.request.body)}

        elif content_type == 'application/json':
            # noinspection PyBroadException
            try:
                body = self.request.body
            except Exception as e:
                raise ExecutionError(400, e)

            try:
                request_json = json.loads(to_unicode(body))
                if self.batch:
                    assert isinstance(request_json, list), (
                        'Batch requests should receive a list, but received {}.'
                    ).format(repr(request_json))
                    assert len(request_json) > 0, (
                        'Received an empty list in the batch request.'
                    )
                else:
                    assert isinstance(request_json, dict), (
                        'The received data is not a valid JSON query.'
                    )
                return request_json
            except AssertionError as e:
                raise HTTPError(status_code=400, log_message=str(e))
            except (TypeError, ValueError):
                raise HTTPError(status_code=400, log_message='POST body sent invalid JSON.')

        elif content_type in ['application/x-www-form-urlencoded', 'multipart/form-data']:
            return self.request.query_arguments

        return {}

    @coroutine
    def get_response(self, data, method, show_graphiql=False):
        query, variables, operation_name, id = self.get_graphql_params(self.request, data)

        execution_result = yield self.execute_graphql_request(
            method,
            query,
            variables,
            operation_name,
            show_graphiql
        )

        status_code = 200
        if execution_result:
            response = {}

            if getattr(execution_result, 'is_pending', False):
                event = Event()
                on_resolve = lambda *_: event.set()  # noqa
                execution_result.then(on_resolve).catch(on_resolve)
                yield event.wait()

            if hasattr(execution_result, 'get'):
                execution_result = execution_result.get()

            if execution_result.errors:
                response['errors'] = [self.format_error(e) for e in execution_result.errors]

            if execution_result.invalid:
                status_code = 400
            else:
                response['data'] = execution_result.data

            if self.batch:
                response['id'] = id
                response['status'] = status_code

            result = self.json_encode(response, pretty=self.pretty or show_graphiql)
        else:
            result = None

        raise Return((result, status_code))

    @coroutine
    def execute_graphql_request(self, method, query, variables, operation_name, show_graphiql=False):
        if not query:
            if show_graphiql:
                raise Return(None)
            raise HTTPError(400, 'Must provide query string.')

        source = Source(query, name='GraphQL request')

        try:
            document_ast = parse(source)
            validation_errors = validate(self.schema, document_ast)
        except Exception as e:
            raise Return(ExecutionResult(errors=[e], invalid=True))

        if validation_errors:
            raise Return(ExecutionResult(
                errors=validation_errors,
                invalid=True,
            ))

        if method.lower() == 'get':
            operation_ast = get_operation_ast(document_ast, operation_name)
            if operation_ast and operation_ast.operation != 'query':
                if show_graphiql:
                    raise Return(None)

                raise HTTPError(405, 'Can only perform a {} operation from a POST request.'
                                .format(operation_ast.operation))

        try:
            result = yield self.execute(
                document_ast,
                root_value=self.root_value,
                variable_values=variables,
                operation_name=operation_name,
                context_value=self.context,
                middleware=self.middleware,
                executor=self.executor or TornadoExecutor(),
                return_promise=True
            )
        except Exception as e:
            raise Return(ExecutionResult(errors=[e], invalid=True))

        raise Return(result)

    @coroutine
    def execute(self, *args, **kwargs):
        raise Return(execute(self.schema, *args, **kwargs))

    def json_encode(self, d, pretty=False):
        if pretty or self.get_query_argument('pretty', False):
            return json.dumps(d, sort_keys=True, indent=2, separators=(',', ': '))

        return json.dumps(d, separators=(',', ':'))

    def render_graphiql(self, query, variables, operation_name, result):
        return render_graphiql(
            query=query,
            variables=variables,
            operation_name=operation_name,
            result=result,
            graphiql_version=self.graphiql_version,
            graphiql_template=self.graphiql_template,
            graphiql_html_title=self.graphiql_html_title,
        )

    def should_display_graphiql(self):
        raw = 'raw' in self.request.query_arguments.keys() or 'raw' in self.request.arguments
        return not raw and self.request_wants_html()

    def request_wants_html(self):
        accept_header = self.request.headers.get('Accept', '')
        accept_mimetypes = parse_accept_header(accept_header, MIMEAccept)
        best = accept_mimetypes.best_match(['application/json', 'text/html'])
        return best == 'text/html' and accept_mimetypes[best] > accept_mimetypes['application/json']

    @property
    def content_type(self):
        return self.request.headers.get('Content-Type', 'text/plain').split(';')[0]

    @property
    def context(self):
        return self.request

    @staticmethod
    def instantiate_middleware(middlewares):
        for middleware in middlewares:
            if inspect.isclass(middleware):
                yield middleware()
                continue
            yield middleware

    def get_graphql_params(self, request, data):
        single_args = {}
        for key in request.arguments.keys():
            single_args[key] = self.decode_argument(request.arguments.get(key)[0])

        query = single_args.get('query') or data.get('query')
        variables = single_args.get('variables') or data.get('variables')
        id = single_args.get('id') or data.get('id')

        if variables and isinstance(variables, six.string_types):
            try:
                variables = json.loads(variables)
            except:  # noqa
                raise HTTPError(400, 'Variables are invalid JSON.')

        operation_name = single_args.get('operationName') or data.get('operationName')
        if operation_name == "null":
            operation_name = None

        return query, variables, operation_name, id

    def handle_error(self, ex):
        if not isinstance(ex, (web.HTTPError, ExecutionError, GraphQLError)):
            tb = ''.join(traceback.format_exception(*sys.exc_info()))
            app_log.error('Error: {0} {1}'.format(ex, tb))
        self.set_status(self.error_status(ex))
        error_json = json_encode({'errors': self.error_format(ex)})
        app_log.debug('error_json: %s', error_json)
        self.write(error_json)

    @staticmethod
    def error_status(exception):
        if isinstance(exception, web.HTTPError):
            return exception.status_code
        elif isinstance(exception, (ExecutionError, GraphQLError)):
            return 400
        else:
            return 500

    @staticmethod
    def error_format(exception):
        if isinstance(exception, ExecutionError):
            return [{'message': e} for e in exception.errors]
        elif isinstance(exception, GraphQLError):
            return [format_graphql_error(exception)]
        elif isinstance(exception, web.HTTPError):
            return [{'message': exception.log_message}]
        else:
            return [{'message': 'Unknown server error'}]

    @staticmethod
    def format_error(error):
        if isinstance(error, GraphQLError):
            return format_graphql_error(error)

        return {'message': six.text_type(error)}
